local tec = TeamEquipCompare;

local function TEC_OnTooltipSetItem(tooltip, ...)
	if (not tec.db.global.requireCtrlKey) or ((tec.db.global.requireCtrlKey) and IsControlKeyDown()) then
		-- get info for tooltip item
		tt_name, tt_link = tooltip:GetItem();
		_, _, _, _, tt_reqLevel, tt_class, tt_subclass, _, tt_equipSlot, _, _ = GetItemInfo(tt_link);
		local slot = TEC_Slot_Lookup[tt_equipSlot];

		-- make sure the slot is one we're interested in
		if (slot and slot.ids) then
			-- add heading line with spacing
			tooltip:AddLine(" ");
			if (not IsInGroup()) then
				-- simple message if player is not in a group
				tooltip:AddDoubleLine("Team Equip Compare:", "|cffff0000[Not in Group]|r");
				tooltip:AddLine(" ");
				return;
			else
				tooltip:AddLine("Team Equip Compare:");
				tooltip:AddLine(" ");
			end
			
			-- loop thru party members in alphabetical order
			local sortedNames = tec_keys(tec.party, true);
			for _, unit in pairs(sortedNames) do
				local class = TEC_Class_Lookup[tec.party[unit].class];
				local classColor = TEC_ClassColor_Lookup[tec.party[unit].class];
				local isUsable = TEC_ItemIsUsable(tec.party[unit], class, tt_reqLevel, tt_class, tt_subclass);
				local playerTag = format("|c%s%s (%d - %s):|r", classColor, tec.party[unit].name, tec.party[unit].level, tec.party[unit].spec);

				if (not isUsable) then
					-- add player name with note that tooltip item is unusable
					tooltip:AddDoubleLine(playerTag, "|cffff0000[Unable to Equip]|r");
					tooltip:AddLine(" ");
				else
					-- add party member heading
					tooltip:AddLine(playerTag);

					-- compare item with each potential slot (weapons, trinkets and fingers have two slots)
					for ii = 1, #slot.ids do
						-- grab item from syncronized party member data
						local link = tec.party[unit].gear[slot.ids[ii]];
						if (link) then
							-- get party member item data
							local name, _, quality, ilevel, _, _, _, _, _, _, _ = GetItemInfo(link);
							if (name) then
								local itemTag = format("    |c%s[%d] %s|r", TEC_QualityColor_Lookup[quality], ilevel, name)
								local statLine1, statLine2 = GetStatLines(isUsable, tt_link, link);
				
								tooltip:AddDoubleLine(itemTag, statLine1);
								if (statLine2) and (#statLine2 > 0) then
									tooltip:AddDoubleLine(" ", statLine2);
								end
							end
						end
					end

					-- add spacer between party members
					tooltip:AddLine(" ");
				end
			end
		end
	end
end

local function TEC_OnTooltipCleared(tooltip, ...)
end

-- function can't be local, presumably since it's called from GlobalTooltip
function TEC_ItemIsUsable(unit, class, reqLevel, itemClass, itemSubclass)
	if (reqLevel > unit.level) then
		return false;
	elseif (not class) then
		return false;
	elseif (not class[itemClass]) then
		return false;
	elseif (not class[itemClass][itemSubclass]) then
		return false;
	elseif (class[itemClass][itemSubclass] > unit.level) then
		return false;
	else
		return true;
	end
end

-- function can't be local, presumably since it's called from GlobalTooltip
function GetStatLines(isUsable, itemLink, equippedLink)
	local line1 = "";
	local line2 = "";

	if (isUsable) then
		local statDelta = GetItemStatDelta(itemLink, equippedLink);
		local count = 0;
		for stat, value in pairs(statDelta) do
			-- get friendly name string from lookup table
			statName = TEC_Stat_Lookup[stat];
			if (not statName) then statName = tostring(stat) end

			-- color comparison text depending on +/-
			if (value > 0) then
				if (#line1 > 0) then line1 = line1 .. ", " end
				line1 = line1..format("|cff00ff00+%d|r %s", value, statName);
				count = count + 1;
			elseif (value < 0) then
				if (#line2 > 0) then line2 = line2 .. ", " end
				line2 = line2..format("|cffff0000%d|r %s", value, statName);
				count = count + 1;
			end
		end

		-- make adjustments based on how much was added
		if (#line1 == 0) and (#line2 == 0) then
			line1 = "[Identical]";
		elseif (#line1 == 0) then
			-- if there are only negative values, shift it up a line
			line1 = line2;
			line2 = "";
		elseif (count < 5) then
			line1 = line1..", "..line2;
			line2 = "";
		end
	end

	return line1, line2;
end

-- hook into the game tooltip
GameTooltip:HookScript("OnTooltipSetItem", TEC_OnTooltipSetItem);
GameTooltip:HookScript("OnTooltipCleared", TEC_OnTooltipCleared);